import YAML from "yaml";

export interface FrontmatterShape {
  tags: string[];
  aliases: string[];
  cssclasses: string[];
  [key: string]: string | string[];
}

const EMPTY_FRONTMATTER: FrontmatterShape = {
  tags: [],
  aliases: [],
  cssclasses: [],
};

const FRONTMATTER_REGEX = /^---[\s\S]*?---/gm;
const TAG_REGEX = /#[A-Za-z0-9/\-_]+/g;

/**
 * Attempt to parse YAML frontmatter from a mixed yaml/md doc
 * @param content The raw markdown content
 * @returns Any successfully parsed frontmatter
 */
export function extractFrontmatter(content: string): FrontmatterShape {
  // If it does not start with `---`, it is invalid for frontmatter
  if (content.trim().indexOf("---") !== 0) {
    return {
      ...EMPTY_FRONTMATTER,
    };
  }

  let frontmatterData = {};

  if (FRONTMATTER_REGEX.test(content)) {
    const frontmatterString = content.match(FRONTMATTER_REGEX)![0];
    const cleanFrontmatter = frontmatterString.replaceAll("---", "").trim();
    frontmatterData = YAML.parse(cleanFrontmatter);
  }

  return {
    ...EMPTY_FRONTMATTER,
    ...frontmatterData,
  };
}

export function extractMarkdown(content: string): string {
  if (content.trim().indexOf("---") !== 0) {
    return content;
  }
  return content.replace(FRONTMATTER_REGEX, "").trim();
}

/**
 * Combine yaml frontmatter and markdown into a single string
 * @param markdown The markdown to combine with frontmatter
 * @param frontmatter The frontmatter shape to combine with markdown
 * @returns A combined document with yaml frontmatter and markdown below
 */
export function combineMarkdownAndFrontmatter(
  markdown: string,
  frontmatter: FrontmatterShape,
) {
  const frontmatterToWrite: Partial<FrontmatterShape> =
    structuredClone(frontmatter);

  if (frontmatterToWrite.aliases?.length === 0) {
    delete frontmatterToWrite.aliases;
  }

  if (frontmatterToWrite.cssclasses?.length === 0) {
    delete frontmatterToWrite.cssclasses;
  }

  if (frontmatterToWrite.tags?.length === 0) {
    delete frontmatterToWrite.tags;
  }

  return `---
${YAML.stringify(frontmatterToWrite)}
---

${markdown.trim()}
`;
}

/**
 * Find hashtags in text content
 * @param content The content to search for tags in
 * @returns An array of discovered tags (hash excluded)
 */
export function extractTags(content: string): string[] {
  const results = content.match(TAG_REGEX);

  if (results === null) {
    return [];
  }

  return results.map((result) => result.replace("#", ""));
}
